#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-lfc-client-config v. 1.5.0
#
# Post-installation script for configuring the gLite lfc client
# Guillermo Diez-Andino Sancho <Guillermo.Sancho@cern.ch>
#
# Version info: $Id: glite-lfc-client-config.py,v 1.2.2.2 2006/05/09 09:03:45 harakaly Exp $
#
# Usage: python glite-lfc-client-config [-c|-v|-h|--help|--start|--stop|--status|--configure]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --configure configures the pbs-lfc client services (pbs_mom)
#        --siteconfig=<site config URL>     use remote site configuration file from URL
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

# import system libraries
import os,string
import sys
import getopt
import time

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams

# Set global variables here 
global params                     # all config values from the XML file

class glite_lfc_client:

    def __init__(self):
        self.verbose = 0
        self.version = "1.5.0"
        self.name = "glite-lfc-client"
        self.friendly_name = "gLite LFC Client"
        self.glib = params['glib']
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure           configures the lfc client '
        print '    --siteconfig        URL of the site configuration file'
        print '\n'

    def start(self):
        return 0

    def stop(self):
        return 0
    
    def status(self):
        return 0
    
    def backup(self,path):
        backupPath = "%s.1" % path

        if os.path.exists(backupPath):
            os.remove(backupPath)
        if os.path.exists(path):
            os.rename(path,backupPath)
        return 0
        
    def existsFile(self,path):
        if os.path.exists(path):
            print "File %s correctly created                 " %path,
            self.glib.printOkMessage()
            return 0
        else:
            print "Problems to create file %s                " %path,
            self.glib.printFailedMessage()
            return 1
    
    
    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------
    
    def configure(self):
        # ------------------------------------------------------------
        # Run security util script
        # ------------------------------------------------------------
                                                                                                                                                                                 
        errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])
                                                                                                                                                                                 
        if errorcode:
            print "Configuring security                                      ",
            glib.printFailedMessage()
            return 1
        
        print "Configuring security                                      ",
        glib.printOkMessage()            
        return 0
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def loadDefaults(params):

    glib.setGlobalDefaultParameters(params)
        
def set_env():

    if not os.geteuid() == 0:
        
        params['installer.export.filename'] = glib.checkSetEnvFile(params['installer.export.filename'])
        glib.printInfoMessage("gLite environment will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/site_perl/5.6.0/" % installPrefix)
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/vendor_perl/5.8.0/" % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
        params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if not os.path.exists(params['GLOBUS_LOCATION']):
            print "    GLOBUS_LOCATION is set to the non-existing directory %s " % (params['GLOBUS_LOCATION']),
            params['GLOBUS_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GLOBUS_LOCATION']

        if not os.path.exists(params['GPT_LOCATION']):
            print "    GPT_LOCATION is set to the non-existing directory %s " % (params['GPT_LOCATION']),
            params['GPT_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GPT_LOCATION']

        if not os.path.exists(params['LCG_LOCATION']):
            print "    LCG_LOCATION is set to the non-existing directory %s " % (params['LCG_LOCATION']),
            params['LCG_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['LCG_LOCATION']

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite LFC Client installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite LFC Client installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
            print "\n"


    # Standard configuration
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('GLOBUS_LOCATION')
    glib.export('GRID_PROXY_FILE', "/tmp/x509up_u`id -u`")
    glib.export('X509_USER_PROXY', "/tmp/x509up_u`id -u`")
    glib.export('GLITE_WMS_LOCATION', params['GLITE_LOCATION'])

    glib.export('LCG_LOCATION')
    glib.export('LFC_HOST',params['lfc.server'])
    glib.export('GPT_LOCATION')
                                                                                                                                                                  
    glib.addEnvPath("PATH","$GLOBUS_LOCATION/bin")
    glib.addEnvPath('PATH','$GLITE_LOCATION/externals/bin')
    glib.addEnvPath("PATH","$LCG_LOCATION/bin")
    glib.addEnvPath('PATH','$GLITE_LOCATION/bin')
    glib.addEnvPath('PATH','$JAVA_HOME/bin')

    glib.addEnvPath("LD_LIBRARY_PATH","$GLOBUS_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$LCG_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","/usr/local/lib")

    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl:%s/lib/perl5" % (os.environ['GPT_LOCATION'],os.environ['GLITE_LOCATION']))

    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLOBUS_LOCATION'] + '/man')
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

    glib.exportToLdSoConf("%s/lib" % os.environ['GLOBUS_LOCATION'])
    glib.exportToLdSoConf("/usr/local/lib")

    glib.setUserEnv()    
                                                                                                                                                                  
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # Load parameters
    params = ConfigParams() 
    
    # Get an instance of the library class
    glib = gLib()

    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass

    params['glib'] = glib

    if glib.loadConfiguration(["%s/../glite-lfc-client.cfg.xml" % glib.getScriptPath()] ,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)
    
    if not os.geteuid() == 0:
        if os.path.exists("pwd.getpwnam(username)[5]/.glite/glite-ui.cfg.xml"):
            glib.loadConfigurationFile("pwd.getpwnam(username)[5]/.glite/glite-ui.cfg.xml",params)

    glib.set_params(params)

    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    if params.has_key('glite.installer.verbose'):
        if (params['glite.installer.verbose'] == "true"):
            verbose=1
        else:
            verbose=0
    else:
        verbose = 0
    glib.verbose = verbose
    
    # Set up the environment
    service = glite_lfc_client()
    set_env()    
                                                                                                  
    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','configure','subservice','siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "[FAILED] Unknown options(s)")
        sys.exit(1)

    # Check cli options
    configure = 0
    subservice = 0
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
        if o == "--subservice":
            subservice = 1        
        if o == "--configure":
            configure =1
    
    if configure == 1:
        if subservice == 0:
            service.copyright()
            service.showVersion()

        # Display the parameters if verbose output
        if verbose:
            glib.print_params(params)
    
        if not os.geteuid() == 0:

            if verbose:
                glib.printInfoMessage("Detected 'User installation'")

            if os.stat(sys.argv[0]).st_uid != os.geteuid():
                if verbose:
                    glib.printInfoMessage("You are not the owner of the configuration file")
                    glib.printInfoMessage("Creating new local user configuration")
                if os.stat(sys.argv[0]).st_uid != 0:
                    params['installer.export.filename'] = pwd.getpwuid(os.stat(sys.argv[0]).st_uid)[5] + "/.glite/glite_setenv.sh"
                if not os.environ.has_key('GLITE_LOCATION'):
                    os.environ['GLITE_LOCATION'] = os.path.abspath(glib.getScriptPath() + "/../../../")
                os.system("cp %s/etc/vomses %s/.vomses" % (os.environ['GLITE_LOCATION'], os.environ['HOME']))
                glib.setUserEnv()
                print "\nThe gLite UI configuration has finished successfully            ",
                glib.printOkMessage()
    
                sys.exit(0)
            else:
                set_env()
                userInst = 1

        else:

            if verbose:
                glib.printInfoMessage("Detected 'Root installation'")
            set_env()
            userInst = 0
            
        # configure service
        if service.configure()==0:
            print "\n%s configuration successfully completed\n" % service.friendly_name
            sys.exit(0)
        else:
            print "\nAn unrecoverable error occurred while configuring the %s service" \
                % service.friendly_name
            sys.exit(1)    
    
    else:        

        # Print configuration parameters
        if verbose:
            glib.print_params(params)
            service.copyright()
            service.showVersion()
            service.banner()
            service.usage()

    sys.exit(0)
    
    